package org.example.base;

import com.google.common.collect.Lists;
import com.google.common.collect.Ordering;
import com.google.common.primitives.Ints;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

// 排序器

public class OrderingGuava {
    public static void main(String[] args) {
        //对可排序类型做自然排序，如数字按大小，日期按先后排序
        Ordering<Comparable> natural = Ordering.natural();
        System.out.println(natural.isOrdered(new ArrayList<Integer>(Arrays.asList(1,4,3))));
        //按对象的字符串形式做字典排序
        Ordering.usingToString();
        //把给定的Comparator转化为排序器或者继承Ordering实现自定义排序器
        Ordering<Integer> from = Ordering.from((Integer x, Integer y) -> Ints.compare(x, y));
        System.out.println(from.max(5, 6));


        //扩展排序器
        Ordering<Comparable> natural1 = Ordering.natural();
        natural1.max(1, 2); //2
        //获取语义相反的排序器
        natural1.reverse().max(1, 2); //1
        List<Integer> list = Lists.newArrayList(5, 9, 3, 7, 4);
        Collections.sort(list,natural1.nullsFirst()); //使用排序器对集合排序
        list.forEach(x-> System.out.print(x+" ")); //3, 4, 5, 7, 9
        System.out.println();

        // 不使用这两个排序器,排序元素不能为null,否则会报空指针
        natural1.nullsFirst();//使用当前排序器，并把null值排到最前
        natural1.nullsLast();//使用当前排序器，并把null值排到最后

        natural1.compound(natural1);//合成另一个比较器，以处理当前排序器中的相等情况,即添加第二个比较器

        //onResultOf:将function应用在各个元素上之后, 在使用原始ordering进行排序
        Collections.sort(list,natural1.onResultOf(x->x*(-1)));//元素变为负数再排序
        list.forEach(x-> System.out.print(x+" ")); //9, 7, 5, 4, 3
        System.out.println();

        // 使用排序器
        Ordering<Comparable> natural3 = Ordering.natural();
        List<Integer> list02 = Lists.newArrayList(6,4,9,3,1,5,3);
        /**
         * greatestOf: 获取可迭代对象中最大的k个元素,并按从大到小排序,返回一个集合
         * leastOf: 获取最小的几个,并按从小到大排序
         */
        natural3.greatestOf(list02,1).forEach(x-> System.out.println(x)); //9
        natural3.leastOf(list02,1).forEach(x-> System.out.println(x)); //1
        //返回集合的一个排好序的副本
        natural3.sortedCopy(list02);
        natural3.immutableSortedCopy(list02); //返回不可变的排序副本
        /**
         * isOrdered: 判断是否已经按排序器有序,元素不能少于2,允许有相等元素
         * isStrictlyOrdered: 是否严格排序,不允许有相等元素
         */
        Collections.sort(list02,natural3);
        natural3.isOrdered(list02);//true
        natural3.isStrictlyOrdered(list02); //false
        //其他直接比较元素
        System.out.println(natural3.compare(1,2));
        System.out.println(natural3.max(5,6));
        System.out.println(natural3.min(list02));
    }
}
