package org.example.io;

import com.google.common.base.CharMatcher;
import com.google.common.base.Charsets;
import com.google.common.base.Splitter;
import com.google.common.collect.HashMultiset;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Multiset;
import com.google.common.hash.HashCode;
import com.google.common.hash.Hashing;
import com.google.common.io.*;

import java.io.File;
import java.io.IOException;
import java.net.URL;

public class IOOperation {
    public static void main(String[] args) throws IOException {
        //guava提供了一个源与汇的概念对应读写字节字符共有4个类ByteSource,CharSource,ByteSink,CharSink
        URL url = new URL("http://localhost:8080/queryAll");
        File file = new File("./temp.txt");
        Resources.toByteArray(url);//从url中获得字节数组
        Files.toByteArray(file);//从文件中获得字节数组
        CharSource charSource = Files.asCharSource(file, Charsets.UTF_8);//获得字符源
        ByteSource byteSource = Files.asByteSource(file);//获得字节源
        Resources.asCharSource(url, Charsets.UTF_8);//从url中获得字符源
        Resources.asByteSource(url);//从url中获得字节源

        CharSink charSink = Files.asCharSink(file, Charsets.UTF_8, FileWriteMode.APPEND);//获得字符汇 以append追加方式（覆盖方式可以省略）
        ByteSink byteSink = Files.asByteSink(file, FileWriteMode.APPEND);//或者字节汇 以append追加方式（覆盖方式可以省略）

        byteSource.asCharSource(Charsets.UTF_8);//字节源转字符源
        byteSink.asCharSink(Charsets.UTF_8);//字节汇转字符汇
        byte[] bytes = byteSource.read();//读取出byte[]
        String string = charSource.read();//读取出String
        byteSource.copyTo(byteSink);//字节源copy到汇 可以使用OutputStream
        charSource.copyTo(charSink);//字符源copy到汇 可以使用Appendable
        byteSink.write(bytes);//写入字节
        charSink.write(string);//写入字符（使用的是CharSequence，可以用String）

//案例
//逐行读取 以utf-8编码
        ImmutableList<String> lines = Files.asCharSource(file, Charsets.UTF_8).readLines();
        System.out.println(lines);

//读取单词
        Multiset<String> wordOccurrences = HashMultiset.create(
                //以空格拆分
                Splitter.on(CharMatcher.whitespace())
                        .trimResults()
                        .omitEmptyStrings()
                        .split(Files.asCharSource(file, Charsets.UTF_8).read()));
        System.out.println(wordOccurrences);

        //获取文件按照sha1生成的hash码
        HashCode hash = Files.asByteSource(file).hash(Hashing.sha1());
        System.out.println(hash);

//File的工具类Files
//将url资源copy到file中
        Resources.asByteSource(url).copyTo(Files.asByteSink(file));
    }
}
