package org.example.collections;

import com.google.common.collect.Range;
import com.google.common.collect.RangeSet;
import com.google.common.collect.TreeRangeSet;
import org.junit.Test;

import java.util.Iterator;

public class RangeSetTest {
    @Test
    public void testRangeSet(){
        RangeSet rangeSet = TreeRangeSet.create();
        rangeSet.add(Range.closed(1, 10));
        System.out.println(rangeSet);   // [[1..10]]

        rangeSet.add(Range.closedOpen(11, 15));
        System.out.println(rangeSet);   // [[1..10], [11..15)]

        rangeSet.add(Range.open(15, 20));
        System.out.println(rangeSet);   // [[1..10], [11..15), (15..20)]

        rangeSet.add(Range.openClosed(0, 0));
        System.out.println(rangeSet);   // [[1..10], [11..15), (15..20)]

        rangeSet.remove(Range.open(5, 10));
        System.out.println(rangeSet);   // [[1..5], [10..10], [11..15), (15..20)]\

        // 遍历:
        // 方法1
        // rangeSet.asDescendingSetOfRanges().forEach(System.out::println);  // 倒序
        Iterator<Range> iterator = rangeSet.asRanges().iterator();
        while(iterator.hasNext()){
            Range next = iterator.next();
            System.out.println(next);
        }

         // 方法2
        rangeSet.asRanges().forEach(System.out::println);

        // 互补范围
        RangeSet complement = rangeSet.complement();
        System.out.println(complement);

        // 包含
        boolean isIn = rangeSet.contains(15);
        System.out.println(isIn);   // false，因为上述范围不包含元素15.

        // 查找范围
        // 元素是在rangeSet中哪个范围里面
        Range integerRange = rangeSet.rangeContaining(17);
        System.out.println(integerRange);   // 输出 `(15..20)`，因为17被包含在 `(15..20)`中 ，所以输出这个范围。
        // 某个范围是否包含在rangeSet的范围中
        boolean encloses = rangeSet.encloses(Range.closedOpen(18, 20));
        System.out.println(encloses);   // true.因为范围(18,20)包含在范围(15,20)中
        encloses = rangeSet.encloses(Range.closedOpen(5, 20));
        System.out.println(encloses);   // false 因为范围(5,20)不被rangeSet中任何范围包含.

        RangeSet<Integer> rangeSet1 = TreeRangeSet.create();
        rangeSet1.add(Range.closed(1, 10));      // {[1,10]}
        rangeSet1.add(Range.closedOpen(11, 15)); // 不相连区间:{[1,10], [11,15)}
        rangeSet1.add(Range.closedOpen(15, 20)); // 相连区间; {[1,10], [11,20)}
        rangeSet1.add(Range.openClosed(0, 0));   // 空区间; {[1,10], [11,20)}
        rangeSet1.remove(Range.open(5, 10));     // 分割[1, 10]; {[1,5], [10,10], [11,20)}


    }

}
