package org.example.functional;

import com.google.common.base.Function;
import com.google.common.base.Functions;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

public class FunctionsPredicatesTest {
    @Test
    public void test1() {
        Map<String, Long> stringLong = new HashMap<String, Long>();
        stringLong.put("key1", 1L);
        stringLong.put("key2", 2L);
        //将Map实例转换为Function实例
        Function<String, Long> stringLongFunction = Functions.forMap(stringLong, 0L);
        // 1
        System.out.println(stringLongFunction.apply("key1"));
        /* 第一个Function是定义转换规则，对第二个Function的value进行处理，
         * 下面栗子里，用新建的Function对stringLongFunction的value进行处理，规则就是重写的apply()方法
         */
        Function<String, String> compose = Functions.compose(new Function<Long, String>() {
            @Override
            public String apply(Long input) {
                return input * input + "";
            }
        }, stringLongFunction);
        Function<String, String> compose1 = Functions.compose(new Function<String, String>() {
            @Override
            public String apply(String input) {
                return input + input;
            }
        }, compose);
        // 44 = "2*2" + "2*2" 注意这个是对上面compose进行的处理
        System.out.println(compose1.apply("key2"));
    }


    @Test
    public void test3() {
        Map<String, Long> stringLong = new HashMap<String, Long>();
        stringLong.put("key1", 1L);
        stringLong.put("key3", 3L);
        Function<String, Long> stringLongFunction = Functions.forMap(stringLong, 0L);
        //true,false
        System.out.println(Predicates.alwaysTrue().apply(1));
        System.out.println(Predicates.isNull().apply(1));

        //多个组合
        Predicate<Object> or = Predicates.or(Predicates.isNull(), Predicates.notNull());
        //true,true
        System.out.println(or.apply(null));
        System.out.println(or.apply("1"));
        Predicate<Long> and = Predicates.and(new Predicate<Long>() {
            @Override
            public boolean apply(Long input) {
                return input < 5;
            }
        }, new Predicate<Long>() {
            @Override
            public boolean apply(Long input) {
                return input > 2;
            }
        });
        Predicate<String> compose = Predicates.compose(and, stringLongFunction);
        System.out.println("--------Predicate----------");
        //false,true
        System.out.println(compose.apply("key1"));
        System.out.println(compose.apply("key3"));

        //转换为Function
        Function<Long, Boolean> longBoolean = Functions.forPredicate(and);
        System.out.println("--------Predicate-Function---------");
        //true,false
        System.out.println(longBoolean.apply(3L));
        System.out.println(longBoolean.apply(6L));

    }
}
